import React, { useEffect } from 'react';
import { Box, Typography, Button, Alert } from '@mui/material';
import styled from 'styled-components';
import LocalGasStationIcon from '@mui/icons-material/LocalGasStation';
import CountdownTimer from './CountdownTimer';

export interface SlotData {
    id: string;
    qr_code: string;
    label: string;
    qr_base: string;
    occupied: boolean;
    dewar_unique_id?: string;
    dewar_name?: string;
    needs_refill?: boolean;
    time_until_refill?: number;
}

interface SlotProps {
    data: SlotData;
    isSelected: boolean;
    onSelect: (data: SlotData) => void;
    onRefillDewar: (qr_code?: string) => Promise<void>;
    reloadSlots: () => Promise<void>;
}

interface StyledSlotProps {
    isSelected: boolean;
    isOccupied: boolean;
}

const StyledSlot = styled(Box)<StyledSlotProps>`
    padding: 16px;
    margin: 8px;
    width: 150px;
    height: 220px;
    background-color: ${({ isSelected, isOccupied }) =>
            isSelected ? '#3f51b5' : isOccupied ? '#f44336' : '#4caf50'};
    color: white;
    cursor: pointer;
    display: flex;
    flex-direction: column;
    justify-content: space-between;
    align-items: center;
    border-radius: 8px;
    box-shadow: 2px 2px 10px rgba(0, 0, 0, 0.1);
    transition: transform 0.2s;

    &:hover {
        transform: scale(1.05);
    }
`;

const Slot: React.FC<SlotProps> = ({ data, isSelected, onSelect, onRefillDewar, reloadSlots }) => {
    useEffect(() => {
        console.log(`Updated time_until_refill: ${data.time_until_refill}`);
    }, [data.time_until_refill]);

    const handleRefill = async () => {
        if (data.dewar_unique_id) {
            await onRefillDewar(data.dewar_unique_id);
            reloadSlots();
        }
    };

    const { id, qr_code, label, qr_base, occupied, needs_refill, time_until_refill, dewar_unique_id, dewar_name, ...rest } = data;

    return (
        <StyledSlot
            isSelected={isSelected}
            isOccupied={occupied}
            onClick={() => onSelect(data)}
            {...rest}
        >
            <Typography variant="h6">{label}</Typography>
            {dewar_name && <Typography variant="body2">{`Dewar: ${dewar_name}`}</Typography>}
            {dewar_unique_id && <Typography variant="body2">{`ID: ${dewar_unique_id}`}</Typography>}
            {needs_refill && <LocalGasStationIcon />}
            {dewar_unique_id && time_until_refill !== undefined && time_until_refill !== -1 && (
                <CountdownTimer key={dewar_unique_id} totalSeconds={time_until_refill} />
            )}
            {needs_refill && (
                <Button onClick={handleRefill} sx={{ mt: 1, color: 'white' }}>
                    Refill
                </Button>
            )}
            {occupied && time_until_refill === -1 && (
                <Alert severity="warning">This dewar has no recorded refill event. It needs to be refilled.</Alert>
            )}
        </StyledSlot>
    );
};

export default Slot;